/* oe_netcdf.h -- Header file for NetCDF functions for optimal estimation library

   Copyright (C) 2005-2006 Robin Hogan <r.j.hogan@reading.ac.uk> 

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <netcdf.h>
#include "oe.h"

#ifdef __cplusplus
extern "C" {
#endif

/* Wrap NC_CHECK() around a netcdf function and if an error occurs,
   call OE_THROW() to report the error. */
extern int ncstatus;
#define NC_CHECK(a) if ((ncstatus = (a)) != NC_NOERR) { \
  OE_THROW(OE_FAILURE, (char *) nc_strerror(ncstatus)); }

/* Read scalars, vectors and matrices into oe-style data structures. */
int oe_read_nc_scalar(int ncid, char *varname, OEreal *value);
int oe_read_nc_vector(int ncid, char *varname, OEvector *v);
int oe_read_nc_matrix(int ncid, char *varname, OEmatrix *M);
int oe_read_nc_slice(int ncid, char *varname, int islice, OEvector *v);
int oe_read_nc_int_slice(int ncid, char *varname, int islice, OEintvector *v);

/* Put an OEmatrix into a NetCDF dataset or an OEvector into a slice
   of a NetCDF matrix. */
int oe_put_nc_matrix(int ncid, int fieldid, OEmatrix *M);
int oe_put_nc_slice(int ncid, int fieldid, int islice, OEvector *v);


#ifdef __cplusplus
}
#endif
