/* oe_debug.c -- Debugging functions for optimal estimation library

   Copyright (C) 2005 Robin Hogan <r.j.hogan@reading.ac.uk> 

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <math.h>

#include "oe.h"

/* Global variables containing backtrace information for error handling */
char *oe_error_message = NULL;
char *oe_error_file_list[OE_MAX_ERROR_NESTING];
int oe_error_line_list[OE_MAX_ERROR_NESTING];
int oe_error_nesting = 0;
int oe_error_code = 0;

int oe_verbose = 0;

/* Print to stderr the elements of a vector that contain
   "not-a-number" */
int
oe_find_nans_in_vector(OEvector *v, char *name)
{
  int i;
  int found = 0;
  for (i = 0; i < v->n; i++) {
    if (isnan(v->data[i])) {
      if (!found) {
	found = 1;
	fprintf(stderr, "Vector \"%s\" contains NaN values at element(s) %d", name, i);
      }
      else {
	fprintf(stderr, ", %d", i);
      }
    }
  }
  if (found) {
    fprintf(stderr, "\n");
    return OE_FAILURE;
  }
  else {
    return OE_SUCCESS;
  }
}

/* Print to stderr the elements of a matrix that contain
   "not-a-number" */
int
oe_find_nans_in_matrix(OEmatrix *M, char *name)
{
  int i, j;
  int found = 0;
  for (j = 0; j < M->m; j++) {
    for (i = 0; i < M->n; i++) {
      if (isnan(M->data[j][i])) {
	if (!found) {
	  found = 1;
	  fprintf(stderr, "Matrix \"%s\" contains NaN values at element(s) [%d,%d]", name, j, i);
	}
	else {
	  fprintf(stderr, ", [%d,%d]", j, i);
	}
      }
    }
  }
  if (found) {
    fprintf(stderr, "\n");
    return OE_FAILURE;
  }
  else {
    return OE_SUCCESS;
  }
}


