/* oe_readconfig.h -- Interface to readconfig structures, containing
   configuration data read from a file or the command line

   Copyright (C) 2005 Robin Hogan <r.j.hogan@reading.ac.uk> 

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "oe.h"

/* Find vector of specified name in the configuration data and return
   it in v */
int
oe_read_vector(rc_data *data, char *name, OEvector *v)
{
  int length;
  OEreal *val = rc_get_real_vector(data, name, &length);
  if (val) {
    v->data = val;
    v->n = length;
    v->_n_allocated = length;
    return OE_SUCCESS;
  }
  else {
    OE_THROW(OE_FAILURE,
	     "Could not find requested vector in configuration data");
  }
}

/* Find matrix of specified name in the configuration data and return
   it in M */
int
oe_read_matrix(rc_data *data, char *name, OEmatrix *M)
{
  int m, n;
  OEreal **val = rc_get_real_matrix(data, name, &m, &n);
  if (val) {
    M->data = val;
    M->m = m;
    M->n = n;
    M->_mn_allocated = m*n;
    M->_m_allocated = m;
    return OE_SUCCESS;
  }
  else {
    OE_THROW(OE_FAILURE,
	     "Could not find requested matrix in configuration data");
  }
}
