/* 
   Copyright (C) 2000 Robin Hogan

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <iostream>
#include <cctype>
#include <cstdio>
#include <string>
#include <cstring>
#include <unistd.h>
#include <sys/types.h>
#include <signal.h>

#ifndef NO_X_FONT_SERVER
#define FONT_CPP
extern "C" {
#include	"FSlib.h"
  /* The FSlib.h header file does not provide proper prototypes 
     for some of its functions in X11R6.3 and earlier */
  extern Font FSOpenBitmapFont(FSServer *, FSBitmapFormat, FSBitmapFormatMask,
			char *, Font *);
  extern int FSQueryXInfo(FSServer *, Font, FSXFontInfoHeader *, FSPropInfo *,
		   FSPropOffset **, unsigned char **);
  extern int FSQueryXExtents16(FSServer *, Font, Bool, FSChar2b *, long unsigned int, FSXCharInfo **);
  extern int FSQueryXBitmaps16(FSServer *, Font, FSBitmapFormat, int, FSChar2b *,
			long unsigned int, FSOffset **, unsigned char **);
  extern int FSCloseFont(FSServer *, Font);
  extern int FSFree(char *);
  extern int FSCloseServer(FSServer *);
  //  extern int (* FSSetErrorHandler(int (* handler) (FSServer *, FSErrorEvent *))) (FSServer *, FSErrorEvent *);
  extern int FSSetErrorHandler(int (* handler) (FSServer *, FSErrorEvent *));

  int _FontErrorHandler(FSServer *svr, FSErrorEvent *event) {
    return 0;
  }
}
#endif

#include "rplot.h"

#ifdef NO_X_FONT_SERVER
/* Build-in fonts */
extern RFont *Helvetica_Medium_R_12;
extern RFont *Helvetica_Medium_R_8;
extern RFont *Helvetica_Medium_O_12;
extern RFont *Helvetica_Bold_R_12;
extern RFont *Helvetica_Bold_O_12;
extern RFont *Helvetica_Medium_R_11;
extern RFont *Helvetica_Medium_O_11;
extern RFont *Helvetica_Bold_R_11;
extern RFont *Helvetica_Bold_O_11;
#endif


bool
font_match(const string &a, const string &b)
{
  unsigned int i;
  if (a == "*" || b == "*" ) {
    return true;
  }
  if (a.length() != b.length()) {
    return false;
  }
  for (i=0;i<a.length();i++) {
    if (tolower(a[i]) != tolower(b[i])) {
      return false;
    }
  }
  return true;
}

RFont::RFont(void)
{
  int i;
  glyphs=NULL;
  left = new int[MAX_CHARS];
  right = new int[MAX_CHARS];
  ascent = new int[MAX_CHARS];
  descent = new int[MAX_CHARS];
  width = new int[MAX_CHARS];
  goffset = new unsigned int[MAX_CHARS];
  glength = new unsigned int[MAX_CHARS];
  exists = new bool[MAX_CHARS];  

  if (!left || !right || !ascent || !descent || !width
      || !goffset || !glength || !exists) {
    throw("\nOut of memory\n");
  }
  for (i=0;i<MAX_FONTS;i++) {
    exists[i]=false;
  }
  static_data = false;
}

#define my_free(name) if (name) delete[] name
RFont::~RFont(void)
{
  if (!static_data) {
    my_free(left);
    my_free(right);
    my_free(ascent);
    my_free(descent);
    my_free(width);
    my_free(goffset);
    my_free(glength);
    my_free(exists);
    if (glyphs != NULL) {
#ifndef NO_X_FONT_SERVER
      FSFree((char *) glyphs);
#else
      throw("Don't know how to deallocate memory");
#endif
    }
  }
}

/*
int RFont::xextent(char *str) {
	int ch=0, ichar, extent=0;
	while (str[ch] != '\0') {
		ichar = (str[ch] < 0 ? (int) 256-str[ch] : str[ch]);
		if (ichar < first || ichar > last) {
			ichar=def;
		}
		if (!exists[ichar-first]) {
			ichar=def;
		}
		extent += width[ichar-first];
		ch++;
	}
	return extent;
}

int RFont::yextent(char *str) {
	return max_ascent + max_descent;
}
*/

void
RFont::show()
{
  int ch, bpr,offset;
  unsigned int char_width, char_height, r, b;

  for (ch=0; ch<=last-first; ch++) {
    if (exists[ch]) {
      cout << "Character " << (int) (ch+first) << ": left "
	   << left[ch] << ", right " 
	   <<	right[ch] << ", ascent " << ascent[ch] << ", descent " 
	   << descent[ch] << ", width " << width[ch] << "\n";
      if (have_glyphs()) {
	char_width = right[ch]-left[ch];
	char_height = ascent[ch]+descent[ch];
	bpr = ((char_width+7)>>3);
	if (glength[ch] != char_height * bpr) {
	  cout << "   Length mismatch\n";
	}
	offset = goffset[ch];
	for (r=0; r<char_height; r++) {
	  unsigned char *row = glyphs + offset;
	  for (b=0; b<char_width; b++) {
	    cout << ((row[b >> 3] & (1 << (7 - (b & 7)))) ? '#' : '-');
	  }
	  cout << "\n";
	  offset += bpr;
	}
      }		
    }
  }
}

RFontLoader::RFontLoader(string server_name, bool require_glyphs)
{
  int i;

  _require_glyphs = require_glyphs;
#ifndef NO_X_FONT_SERVER
  _xfs_pid = 0;
  _server = NULL;
  if (server_name.empty()) {
    _server_name="";
  }
  else {
    _server_name=server_name;
  }
#endif
  for (i=0;i<MAX_FONTS;i++) {
    _font[i]=NULL;
  }
#ifdef NO_X_FONT_SERVER
  _font[0] = Helvetica_Medium_R_12;
  _font[1] = Helvetica_Medium_O_12;
  _font[2] = Helvetica_Bold_R_12;
  _font[3] = Helvetica_Medium_O_12;
  _font[4] = Helvetica_Medium_R_8;
  _font[5] = Helvetica_Medium_R_11;
  _font[6] = Helvetica_Medium_O_11;
  _font[7] = Helvetica_Bold_R_11;
  _font[8] = Helvetica_Medium_O_11;
  _font[0]->parent = _font[1]->parent = _font[2]->parent
    = _font[3]->parent = _font[4]->parent = _font[5]->parent
    = _font[6]->parent = _font[7]->parent = _font[8]->parent 
    = this;
#endif
}

RFontLoader::~RFontLoader()
{
  int i;
  for (i=0;i<MAX_FONTS;i++) {
    if (_font[i]) {
      delete _font[i];
      _font[i] = NULL;
    }
  }
#ifndef NO_X_FONT_SERVER
  close_server();
  if (_xfs_pid) {
    kill(_xfs_pid, SIGTERM);
  }
#endif
}

RFont *RFontLoader::load(int pixel, string family,
			 string weight, string slant, string foundry) {
  int i, first_empty=-1;
  for (i=0;i<MAX_FONTS;i++) {
    if ( _font[i] != NULL ) {
      if ( font_match( _font[i]->family, family) && 
	   font_match( _font[i]->weight, weight) && 
	   font_match( _font[i]->slant, slant) && 
	   font_match( _font[i]->foundry, foundry) && 
	   _font[i]->pixel == pixel &&
	   (_font[i]->have_glyphs() || !_require_glyphs) ) {
	/*
	  if ( _require_glyphs ) {
	  if ( ! _font[i]->have_glyphs() ) {
	  #ifndef NO_X_FONT_SERVER
	  return load_from_server(_font[i], true, pixel,
	  family, weight, slant, foundry);
	  #else
	  return NULL;
	  #endif
	  }
	  }
	*/
	return _font[i];
      }
    }
    else if (first_empty == -1) {
      first_empty = i;
    }
  }
  if (first_empty == -1) {
    return NULL;
  }
  else {
#ifndef NO_X_FONT_SERVER
    _font[first_empty] = load_from_server(NULL, pixel, family, weight, 
					  slant, foundry);
    if (_verbose) {
      if (_font[first_empty] != NULL) {
	cerr << "Loaded font " << _font[first_empty]->foundry << "/" 
	     << _font[first_empty]->family << "/"
	     << _font[first_empty]->weight << "/" 
	     << _font[first_empty]->slant << "/" 
	     << _font[first_empty]->pixel << "px into slot " 
	     << (int) first_empty << " (requested " << pixel << "px)\n";
      }
      else {
	cerr << "Failed to load " << foundry << "/" << family << "/" 
	     << weight << "/" << slant << "/" << (int) pixel << "\n";
      }
    }
    return _font[first_empty];
#else
    // Find nearest of build-in fonts
    for (i=0;i<MAX_FONTS;i++) {
      if ( _font[i] != NULL ) {
	if ( font_match( _font[i]->weight, weight) && 
	     font_match( _font[i]->slant, slant)) {
	  return _font[i];
	}
      }
    }
    if (_font[0]) {
      return _font[0];
    }
    return NULL;
#endif
  }
}

int
RFontLoader::unload(RFont *font)
{
  int i, status=-1;
  for (i=0;i<MAX_FONTS;i++) {
    if (_font[i] == font) {
      delete _font[i];
      _font[i] = NULL;
      status=0;
    }
  }
  return status;
}

#ifndef NO_X_FONT_SERVER

int
RFontLoader::open_server(string server_name)
{
  if (server_name.empty()) {
    /* Normal invocation */
    if (_server_name.empty()) {
      /* Try default font servers */
      if (open_server(string("unix/:-1"))) {
	if (open_server(string("unix/:7100"))) {
	  if (open_server(string("unix/:7101"))) {
	    if (open_server(string("unix/:7102"))) {
	      /* Try to fork an xfs process */
	      pid_t pid = fork();
	      if (pid == -1) {
		/* Fork failed */
		throw("\nUnable to contact font server (tried unix/:-1, 7100, 7101 and 7102) or fork a new process");
	      }
	      else if (pid == 0) {
		/* In child process */
		if (execlp("xfs", "xfs", "-port", "7102", NULL) == -1) {
		  /* Exec failed */
		  cerr << "Failed to start an X font server (xfs)\n";
		  exit(0);
		}
	      }
	      else {
		/* In parent process */
		_xfs_pid = pid;
		sleep(1);
		if (open_server(string("unix/:7102"))) {
		  kill(pid, SIGTERM);
		  _xfs_pid = 0;
		  throw("\nUnable to contact font server (tried unix/:-1, 7100, 7101 and 7102) or contact a new one");
		}
		else if (_verbose) {
		  cerr << "\nStarted X font server (unix/:7102)\n";
		}
	      }
	    }
	  }
	}
      }
    }
    else {
      /* Use font server specified when class was initialised */
      if (open_server(_server_name)) {
	throw("\nUnable to contact font server");
      }
    }
  }
  else {
    /* Recursive invocation: try this exact server */
    char *buf = strdup(server_name.c_str());
    if (!buf) {
      cerr << "\nOut of memory\n";
      return 1;
    }
    FSSetErrorHandler(_FontErrorHandler);
    if ((_server = FSOpenServer(buf)) == NULL) {
      free(buf);
      return 1;
    }
    if (buf) {
      free(buf);
    }
    _server_name=server_name;
    if (_verbose) {
      cerr << "Contacted font server \"" << _server_name << "\"\n";
    }
  }
  return 0;
}

void
RFontLoader::close_server()
{
  if (_server != NULL ) {
    FSCloseServer(_server);
  }
  _server = NULL;
  if (_verbose) {
    cerr << "Closed connection to font server \"" << _server_name << "\"\n";
  }
};

RFont *
RFontLoader::load_from_server(RFont *font, int pixel, string &family, 
			       string &weight, string &slant, string 
			       &foundry)
{
  RFont *target=font;
  Font xfont, dummy_xfont;
  FSXFontInfoHeader header;
  FSXCharInfo *extents;
  FSPropInfo  pi;
  FSPropOffset *po=NULL, *po_const;
  FSChar2b chars[2];
  FSOffset *offsets=NULL;
  FSBitmapFormat format = (BitmapFormatScanlinePad8 
			   | BitmapFormatScanlineUnit8 
			   | BitmapFormatImageRectMin 
			   | BitmapFormatBitOrderMSB 
			   | BitmapFormatByteOrderMSB);
  FSBitmapFormatMask fmask = (BitmapFormatMaskByte 
			      | BitmapFormatMaskBit
			      | BitmapFormatMaskScanLineUnit 
			      | BitmapFormatMaskImageRectangle 
			      | BitmapFormatMaskScanLinePad);
  int char_width, char_height;
  uchar *pd=NULL;
  unsigned short last, ch,i;
  char name[128];
  string property, value;

  if ( _server == NULL) {
    if (open_server()) {
      return NULL;
    }
  }
  //  sprintf(name,"-%s-%s-%s-%s-*-*-%d-*-*-*-*-*-*-*",
  sprintf(name,"*%s-%s-%s-%s-*-*-%d*",
	  foundry.c_str(), family.c_str(), weight.c_str(), 
	  slant.c_str(), pixel);
  xfont = FSOpenBitmapFont(_server, format, fmask, name, &dummy_xfont);
  if (!xfont) {
    return NULL;
  }
  FSQueryXInfo(_server, xfont, &header, &pi, &po, &pd);
  po_const=po;
  FSQueryXExtents16(_server, xfont, True, chars, 0, &extents);
  if (target == NULL) {
    if ((target = new RFont()) == NULL) {
      throw("Cannot allocate memory for font");
    }
  }
  target->parent = this;
  target->first = header.char_range.min_char.low + 
    (header.char_range.min_char.high << 8);
  last = header.char_range.max_char.low + 
    (header.char_range.max_char.high << 8);
  if (last - target->first >= MAX_CHARS) {
    target->last = target->first + MAX_CHARS - 1;
  }
  else {
    target->last = last;
  }
  target->def = header.default_char.low + 
    (header.default_char.high << 8);
  target->max_descent = header.max_bounds.descent;
  target->max_ascent = header.max_bounds.ascent;

  if (_require_glyphs) {
    if (FSQueryXBitmaps16(_server, xfont, format, True, chars, 0,
			  &offsets, &(target->glyphs)) != FSSuccess) {
      throw("Unable to load glyphs");
    };
  }
  for (ch=0; ch<=(target->last - target->first); ch++) {
    target->exists[ch] = true;
    target->left[ch] = extents[ch].left;
    target->right[ch] = extents[ch].right;
    target->ascent[ch] = extents[ch].ascent;
    target->descent[ch] = extents[ch].descent;
    target->width[ch] = extents[ch].width;
    char_width = extents[ch].right-extents[ch].left;
    char_height = extents[ch].ascent+extents[ch].descent;
    if (_require_glyphs) {
      target->goffset[ch]=offsets[ch].position;
      target->glength[ch]=offsets[ch].length;
    }
  }

  target->slant = target->weight = target->foundry = target->family
    = target->xname = target->psname = "";
  target->pixel=0;

  for (i=0; i<pi.num_offsets; i++, po++) {
    property.assign((char *) pd + po->name.position, po->name.length);
    switch (po->type) {
    case PropTypeString:
      if (property == "FOUNDRY") {
	target->foundry.assign((const char *) pd+po->value.position,
			       po->value.length);
      }
      else if (property == "FAMILY_NAME") {
	target->family.assign((const char *) pd+po->value.position,
			      po->value.length);
      }
      else if (property == "SLANT") {
	target->slant.assign((const char *) pd+po->value.position,
			     po->value.length);
      }
      else if (property == "WEIGHT_NAME") {
	target->weight.assign((const char *) pd+po->value.position,
			      po->value.length);
      }
      else if (property == "FONT") {
	target->xname.assign((const char *) pd+po->value.position,
			     po->value.length);
      }
      else if (property == "_DEC_DEVICE_FONTNAMES") {
	target->psname.assign((const char *) pd+po->value.position,
			      po->value.length);
	if (target->psname.substr(0,3) == "PS=") {
	  target->psname = target->psname.substr(3, target->psname.size());
	}
	else {
	  target->psname = "";
	}
      }
      break;
    case PropTypeUnsigned:
      if (property == "PIXEL_SIZE") {
	target->pixel = (unsigned long) po->value.position;
      }
      break;
    case PropTypeSigned:
      if (property == "PIXEL_SIZE") {
	target->pixel = (long) po->value.position;
      }
      break;
    }
  }
  if (target->foundry.empty()) { target->foundry = foundry; };
  if (target->family.empty()) { target->family = family; };
  if (target->weight.empty()) { target->weight = weight; };
  if (target->slant.empty()) { target->slant = slant; };
  if (target->xname.empty()) { target->xname = "Helvetica"; };
  if (target->psname.empty()) { target->psname = "Helvetica"; };
  if (target->pixel == 0) { target->pixel = pixel; };

  FSCloseFont(_server, xfont);
  FSFree((char *) extents);
  FSFree((char *) po_const);
  FSFree((char *) pd);
  if (!_require_glyphs) {
    FSFree((char *) offsets);
  }
  return target;
}
#endif

