/* 
   Copyright (C) 2004 Robin Hogan

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <string>
#include <fstream>
#include <cstdlib>
#include <cstdio>
#include "rplot.h"

//#include "../helvetica_medium_r_adobe.h"

int
RFont::save_to_cpp_file(char *filename)
{
  char font_size[8];
  sprintf(font_size, "%d", pixel);

  ofstream file;
  string data_root = family + "_" + weight + "_" + slant + "_" + font_size;

  if (filename) {
    file.open(filename);
  }
  else {
    string file_name = data_root + ".cpp";
    file.open(file_name.c_str());
  }
  if (!file) {
    throw("Unable to open header file for writing");
  }
  file << "#include \"rplot.h\"\n";

  /* Glyphs */
  int num_chars = MAX_CHARS-1;
  while (num_chars && !exists[num_chars]) {
    num_chars--;
  }
  int length_glyph_data = goffset[num_chars] + glength[num_chars];

  file << "static unsigned char " << data_root << "_glyphs[] = {"
       << std::hex;
  for (int i = 0; i < length_glyph_data-1; i++) {
    file << "0x" << (int) glyphs[i] << ", ";
  }
  file << "0x" << (int) glyphs[length_glyph_data-1]
       << std::dec << "};\n";

#define add_member(member) file << "  " << member << ",\n";
#define add_array(member) file << "  " << data_root << "_" << member << ",\n";
#define add_array_end(member) file << "  " << data_root << "_" << member;
#define add_string(member) file << "  \"" << member << "\",\n"
#define write_array(varname, member, type) file << "static " type " " \
  << data_root << "_" << varname << "[] = {"; \
  for (int i = 0; i < MAX_CHARS-1; i++) { file << member[i] << ", "; } \
  file << member[MAX_CHARS-1] << "};\n"

  write_array("left", left, "int");
  write_array("right", right, "int");
  write_array("ascent", ascent, "int");
  write_array("descent", descent, "int");
  write_array("width", width, "int");

  write_array("goffset", goffset, "unsigned int");
  write_array("glength", glength, "unsigned int");

  write_array("exists", exists, "bool");

  file << "RFont *" << data_root << " = new RFont(\n";

  add_member(pixel);
  add_array("left");
  add_array("right");
  add_array("ascent");
  add_array("descent");
  add_array("width");
  add_member(max_ascent);
  add_member(max_descent);
  add_array("goffset");
  add_array("glength");
  add_string(family);
  add_string(weight);
  add_string(slant);
  add_string(foundry);
  add_string(xname);
  add_string(psname);
  add_member(first);
  add_member(last);
  add_member(def);
  add_array("exists");
  add_array_end("glyphs");

  file << ");\n";

  file.close();
  return 0;
}
