/* 
   Copyright (C) 2000 Robin Hogan

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <string>
#include <fstream>
extern "C" {
#include <png.h>
}
#include "rplot.h"

ostream *png_ostream;

void
cpp_write_png(png_structp png_ptr,
	      png_bytep data, png_uint_32 length)
{
  png_ostream->write((const char *)data,length);
}

void
cpp_flush_png(png_structp png_ptr)
{
  png_ostream->flush();
}

void
RFigure::write_png(ostream *os)
{
  png_structp png_ptr;
  png_infop info_ptr;
  int width,height,bit_depth=8;
  voidp write_io_ptr;
  width=_width;
  height=_height;

#ifdef DEBUG
  cerr << "Creating write structure\n";
#endif

  png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING,
				    NULL,NULL,NULL);
 
  if (png_ptr == NULL)
    {
      throw("Unable to create png write structure");
    }

#ifdef DEBUG
  cerr << "Allocating the image information data\n";
#endif

  /* Allocate/initialize the image information data.  REQUIRED */
  info_ptr = png_create_info_struct(png_ptr);
  if (info_ptr == NULL)
    {
      png_destroy_write_struct(&png_ptr,  (png_infopp)NULL);
      throw("Unable to alocate png data");
    }

#ifdef DEBUG
  cerr << "Set error handling" << endl;
#endif

  /* Set error handling.  REQUIRED if you aren't supplying your own
   * error hadnling functions in the png_create_write_struct() call.
   */
  if (setjmp(png_ptr->jmpbuf))
    {
      png_destroy_write_struct(&png_ptr,  (png_infopp)NULL);
      throw("Unable to set png error handling");
    }

#ifdef DEBUG
  cerr << "Set output control\n";
#endif

  /* set up the output control if you are using standard C streams */
  //   png_init_io(png_ptr, fp);

  /* Reset functions so that C++ streams can be used */
  write_io_ptr = png_get_io_ptr(png_ptr);
  png_ostream=os;
  png_set_write_fn((png_structp) png_ptr,
		   (voidp) write_io_ptr, (png_rw_ptr) cpp_write_png,
		   (png_flush_ptr) cpp_flush_png);

  /* Set the image information here. */

#ifdef DEBUG
  cerr << "Setting image info\n";
#endif
 

  switch (_format) {
  case BITMAP_8BIT:
    /* set the palette if there is one.  REQUIRED for indexed-color images */

    png_set_IHDR(png_ptr, info_ptr, width, height, bit_depth, 
		 PNG_COLOR_TYPE_PALETTE, PNG_INTERLACE_NONE, 
		 PNG_COMPRESSION_TYPE_BASE, PNG_FILTER_TYPE_BASE);

    png_color map[256];
    int i;
    for (i=0;i<=palette._max;i++) {
      map[i].red=palette._color[i].red;
      map[i].green=palette._color[i].green;
      map[i].blue=palette._color[i].blue;
    }
    png_set_PLTE(png_ptr, info_ptr, (png_colorp) map, palette._max+1);
    break;

  case BITMAP_24BIT:

    png_set_IHDR(png_ptr, info_ptr, width, height, bit_depth, 
		 PNG_COLOR_TYPE_RGB, PNG_INTERLACE_NONE, 
		 PNG_COMPRESSION_TYPE_BASE, PNG_FILTER_TYPE_BASE);
    break;

  case BITMAP_32BIT:

    png_set_IHDR(png_ptr, info_ptr, width, height, bit_depth, 
		 PNG_COLOR_TYPE_RGB_ALPHA, PNG_INTERLACE_NONE, 
		 PNG_COMPRESSION_TYPE_BASE, PNG_FILTER_TYPE_BASE);
    break;

  default:
    //         throw("Attempt to write png file of non-bitmap graph (format ="));
    throw("Attempt to write png file of non-bitmap graph");
  }
  /*   png_set_gAMA(png_ptr, info_ptr, gamma);*/

  /* Optionally write comments into the image */
  /*   png_set_text(png_ptr, info_ptr, text_ptr, 3);*/

#ifdef DEBUG
  cerr << "Writing header information\n";
#endif
 
  /* Write the file header information.  REQUIRED */
  png_write_info(png_ptr, info_ptr);

#ifdef DEBUG
  cerr << "Writing image\n";
#endif
 
  png_write_image(png_ptr, (png_bytep *) _data);

#ifdef DEBUG
  cerr << "Cleaning up\n";
#endif
 
  png_write_end(png_ptr, info_ptr);
  png_destroy_write_struct(&png_ptr, (png_infopp)NULL);

  png_ostream=NULL;
}

void
RFigure::write_png(char *filename)
{
  ofstream file;
  file.open(filename);
  if (!file) {
    throw("Unable to open png file for writing");
  }
  write_png(&file);
  file.close();
}

