#!/bin/bash

# This script demonstrates the running of the algorithm on the cases
# used in the paper by Hogan (2007).  Note that the settings might not
# be exactly the same as in the paper so perfect agreement cannot be
# guaranteed.

# Location of the executables
zdr_phidp=../zdr_phidp/zdr_phidp
chilncplot="../rplot/chilncplot -mag 0.75 -stdout"

# First run the retrievals
echo "### REPRODUCING THE CASES OF HOGAN (2007)"
echo "### RUNNING RAIN-ONLY CASE (FIGS 4-6)"
$zdr_phidp settings_no-hail.cfg input=camra_chilbolton_20000818_095450.nc

echo "### RUNNING RAIN-ONLY WITH ONLY ZDR (FIG 7)"
$zdr_phidp settings_Zdr-only.cfg input=camra_chilbolton_20000818_095450.nc

echo "### RUNNING RAIN-ONLY WITH ONLY PHI-DP (FIG 7)"
$zdr_phidp settings_phidp-only.cfg input=camra_chilbolton_20000818_095450.nc

echo "### RUNNING RAIN-ONLY WITH EXTRA 0.5 dB ERROR (FIG 9)"
$zdr_phidp settings_no-hail.cfg input=camra_chilbolton_20000818_095450_errZdr+0.5dB.nc

echo "### RUNNING RAIN-ONLY WITH TESTUD-TYPE ZPHI ALGORITHM (FIG 9)"
$zdr_phidp settings_Testud-ZPHI.cfg input=camra_chilbolton_20000818_095450_errZdr+0.5dB.nc

echo "### RUNNING FIRST HAIL CASE (FIGS 10-12)"
$zdr_phidp settings.cfg input=camra_chilbolton_19990519_172131.nc

echo "### RUNNING SMYTH ET AL. HAIL CASE (FIG 13)"
$zdr_phidp settings.cfg input=camra_chilbolton_19950710_170736.nc

if [ -x ../rplot/chilncplot ]
then
# Now plot the input data
echo "### PLOTTING THE INPUT DATA"
$chilncplot camra_chilbolton_20000818_095450.nc > camra_chilbolton_20000818_095450.png
$chilncplot camra_chilbolton_20000818_095450_errZdr+0.5dB.nc \
    > camra_chilbolton_20000818_095450_errZdr+0.5dB.png
$chilncplot camra_chilbolton_19990519_172131.nc > camra_chilbolton_19990519_172131.png
$chilncplot camra_chilbolton_19950710_170736.nc > camra_chilbolton_19950710_170736.png

# Now plot the retrievals
echo "### PLOTTING THE RETRIEVALS"
$chilncplot camra_chilbolton_20000818_095450_out.nc \
    > camra_chilbolton_20000818_095450_out.png
$chilncplot camra_chilbolton_20000818_095450_out-Zdr-only.nc \
    > camra_chilbolton_20000818_095450_out-Zdr-only.png
$chilncplot camra_chilbolton_20000818_095450_out-phidp-only.nc \
    > camra_chilbolton_20000818_095450_out-phidp-only.png
$chilncplot camra_chilbolton_20000818_095450_errZdr+0.5dB_out-Testud-ZPHI.nc \
    > camra_chilbolton_20000818_095450_errZdr+0.5dB_out-Testud-ZPHI.png
$chilncplot camra_chilbolton_19990519_172131_out.nc \
    > camra_chilbolton_19990519_172131_out.png
$chilncplot camra_chilbolton_19950710_170736_out.nc \
    > camra_chilbolton_19950710_170736_out.png

else
    echo 'Unable to plot results: executable "chilncplot" not found'
fi
