
#include "zdr_phidp.h"

int
find_hail(Settings *settings, Scan *scan, Retrieval *retrieval)
{
  OEreal Zdr_missing_value = scan->Zdr.missing_value;
  int iray = 0;
  int ngates = scan->Zh.n;
  int nrays = scan->Zh.m;

  if (scan->Zdr.n != ngates || scan->Zdr.m != nrays
      || retrieval->Zdr_fwd.n != ngates || retrieval->Zdr_fwd.m != nrays
      || retrieval->Ah.n != ngates || retrieval->Ah.m != nrays) {
    OE_THROW(OE_DIMENSION_ERROR, "Input matrix dimensions do not match");
  }

  OE_CHECK(oe_reset_int_matrix(&retrieval->hail_location, nrays, ngates+1));

  for (iray = 0; iray < nrays; iray++) {
    OEreal *Zdr = scan->Zdr.data[iray];
    OEreal *Zdr_fwd = retrieval->Zdr_fwd.data[iray];
    int *nhailgates = retrieval->hail_location.data[iray];
    int *hailgatelist = nhailgates+1;
    int igate;
    int nconsecutive = 0;
    *nhailgates = 0;
    for (igate = 0; igate < ngates; igate++) {
      if (Zdr[igate] != Zdr_missing_value
	  && Zdr_fwd[igate] - Zdr[igate] > settings->min_hail_Zdr_diff
	  && scan->Zh.data[iray][igate] + retrieval->Ah.data[iray][igate]
	  > settings->min_hail_Zh) {
	hailgatelist[(*nhailgates)++] = igate;
	nconsecutive++;
      }
      else {
	if (nconsecutive 
	    && nconsecutive < settings->min_consecutive_hail_gates) {
	  (*nhailgates) -= nconsecutive;
	}
	nconsecutive = 0;
      }
    }
  }
  return OE_SUCCESS;
}
