#include <stdio.h>
#include "zdr_phidp.h"

static
char *
get_output_filename(char *input, char *suffix)
{
  int i;
  int len_suffix = strlen(suffix);
  int len_base = strlen(input);
  int len = len_suffix + len_base;
  char *output = malloc((len+2)*sizeof(char));
  if (!output) {
    fprintf(stderr, "Error: out of memory\n");
    exit(OE_MEMORY_ERROR);
  }
  strcpy(output, input);
  output[len_base-3] = '_';
  for (i = 0; i < len_suffix; i++) {
    output[i+len_base-2] = suffix[i];
  }
  i += len_base-2;
  output[i++] = '.';
  output[i++] = 'n';
  output[i++] = 'c';
  output[i] = '\0';
  return output;
}



int
main(int argc, char **argv)
{
  Scan scan;
  ScatteringData scattering;
  ScatteringLUT lut;
  Settings settings = DEFAULT_SETTINGS;
  Retrieval retrieval;
  OEreal var_ln_a_apriori;
  OEreal var_atten_scaling;
  int do_hail_retrieval = 0;

  fprintf(stderr, "*** Loading settings\n");
  if (load_settings(argc, argv, &settings) == OE_FAILURE) {
    exit(OE_FAILURE);
  }

  clear_struct(scan);
  clear_struct(scattering);
  clear_struct(lut);
  clear_struct(retrieval);

  /* Load the lookup table */
  fprintf(stderr, "*** Loading scattering lookup table %s\n",
	  settings.lut_filename);
  OE_CATCH(load_scattering(settings.lut_filename, &scattering));
  OE_CATCH(create_scattering_lut(&scattering,
				 settings.distribution_num_D0,
				 settings.distribution_min_D0,
				 settings.distribution_interval_D0,
				 settings.distribution_mu,
				 &lut));
  free_scattering(&scattering);

  OE_CATCH(load_scan(settings.input, &scan));
  /*
  settings.a_default = 200.0;
  settings.b = 1.6;

  settings.azim_decorr_factor = 0.00005; //0.0005;
  settings.var_ln_a_apriori = 1;
  settings.n_basis = 30;
  settings.min_hail_Zh = 35.0;
  settings.min_hail_Zdr_diff = 1.5;
  settings.min_consecutive_hail_gates = 4;
  settings.atten_scaling = 1.0;
  settings.var_atten_scaling = -0.1;
  settings.max_atten = 10.0;
  settings.atten_scaling_decorr = 0.1;

  settings.explicit_range_smoothing = 0;
  if (settings.explicit_range_smoothing) {
    settings.range_decorr_factor = 5.0;
  }
  else {
    settings.range_decorr_factor = 1.0;//10.0;
  }
  */

  if (settings.do_hail_retrieval) {
    do_hail_retrieval = 1;
    settings.do_hail_retrieval = 0;
  }

  /* Save "standard" var_ln_a_apriori */
  var_ln_a_apriori = settings.var_ln_a_apriori;
  var_atten_scaling = settings.var_atten_scaling;

  if (do_hail_retrieval) {
    /* First pass to find hail regions: increase Zdr error */
    OE_CATCH(oe_scale_matrix(&scan.Zdr_error,
			     settings.Zdr_prehail_error_factor
			     * settings.Zdr_error_factor));
    /* Use alternative var_ln_a_apriori for initial pass */
    settings.var_ln_a_apriori = settings.prehail_var_ln_a_apriori;
    settings.var_atten_scaling = -1.0;
  }
  else if (settings.Zdr_error_factor != 1.0) {
    /* Otherwise just scale the Zdr error if this has been
       requested */
    OE_CATCH(oe_scale_matrix(&scan.Zdr_error, settings.Zdr_error_factor));
  }

  if (settings.phi_dp_error_factor != 1.0) {
    /* Scale the phi dp error if this has been requested */
    OE_CATCH(oe_scale_matrix(&scan.phi_dp_error,
			     settings.phi_dp_error_factor));
  }

  scan.is_phi_dp = (settings.use_phi_dp != 0);
  scan.is_Zdr    = (settings.use_Zdr != 0);
  fprintf(stderr, "*** ");
  OE_CATCH(zdr_phidp(&settings, &lut, &scan, &retrieval));

  if (do_hail_retrieval) {
    /* Second pass: find hail, reduce Zdr error and rerun retrieval*/
    if (settings.save_first_pass) {
      if (!settings.first_pass_output) {
	settings.first_pass_output = get_output_filename(settings.input,
					 settings.first_pass_output_suffix);
	fprintf(stderr, "*** Saving first pass to %s\n",
		settings.first_pass_output);
	OE_CATCH(save_retrieval(settings.first_pass_output,
				&retrieval, &scan, &settings));
      }
    }

    fprintf(stderr, "*** Finding hail\n");
    OE_CATCH(find_hail(&settings, &scan, &retrieval));
    settings.do_hail_retrieval = 1;
    OE_CATCH(oe_scale_matrix(&scan.Zdr_error,
			     1.0/settings.Zdr_prehail_error_factor));
    settings.var_ln_a_apriori = var_ln_a_apriori;
    settings.var_atten_scaling = var_atten_scaling;
    fprintf(stderr, "*** ");
    OE_CATCH(zdr_phidp(&settings, &lut, &scan, &retrieval));
  }
  //  OE_CATCH(oe_print_int_matrix(stdout, &retrieval.hail_location, "% hail_location"));

  if (!settings.output) {
    settings.output = get_output_filename(settings.input,
					  settings.output_suffix);
    /*
    int i;
    int len_suffix = strlen(settings.output_suffix);
    int len_base = strlen(settings.input);
    int len = len_suffix + len_base;
    settings.output = malloc((len+2)*sizeof(char));
    if (!settings.output) {
      fprintf(stderr, "Error: out of memory\n");
      exit(OE_MEMORY_ERROR);
    }
    strcpy(settings.output, settings.input);
    settings.output[len_base-3] = '_';
    for (i = 0; i < len_suffix; i++) {
      settings.output[i+len_base-2] = settings.output_suffix[i];
    }
    i += len_base-2;
    settings.output[i++] = '.';
    settings.output[i++] = 'n';
    settings.output[i++] = 'c';
    settings.output[i] = '\0';
    */
  }
  fprintf(stderr, "*** Saving %s\n", settings.output);

  OE_CATCH(save_retrieval(settings.output, &retrieval, &scan, &settings));

  /*  OE_CATCH(oe_save_matrix("Zh.dat", &scan.Zh, "% Zh"));*/
  exit(0);
}
